use strict;
use warnings;
use utf8;

use feature qw(say);

# このスクリプトは不正なリクエストパラメータを弾く例です
# 「型制約で不正なリクエストパラメータを弾く」に対応しています

# --- ブログの制約を作っておく
package Blog;
use Smart::Args;
use Mouse::Util::TypeConstraints;

enum 'Blog::Status' => [qw(public private)];

# Plack::Reuqestの拡張
package My::Request;
use parent qw(Plack::Request);
use Mouse::Util::TypeConstraints;

sub typed_param {
    my ($self, $key, $type) = @_;

    # リクエストパラメータがなければundefを返す
    my $val = $self->parameters->get($key) // return undef;

    # パラメータが制約を満たすならそのパラメータを、
    # 満たさないならundefを返す
    return find_type_constraint($type)->check($val)
        ? $val : undef;
}

package main;

# PSGIの$envを模倣して、クエリパラメータが与えられたようなリクエストを作る

# (1)を使えばvalid requestと表示され、(2)を使えばinvalid requestと表示される
# (1)
my $req = My::Request->new({
    QUERY_STRING => 'blog_status=public',
});
# (2)
# my $req = My::Request->new({
#     QUERY_STRING => 'blog_status=evil',
# });

my $status = $req->typed_param(
    'blog_status', 'Blog::Status',
);
if (! defined $status) {
    say 'invalid request';
}
else {
    say "$status is valid request";
}
