use strict;
use warnings;
use utf8;

use feature qw(say);

# このスクリプトは型定義のサンプルコードです
# 3節のコードに対応しています


use Mouse::Util::TypeConstraints;
use Email::Valid::Loose;
use Smart::Args;

# --- 自然数を定義
subtype 'Natural'
    => as 'Int'
    => where { $_ > 0 }
    => message { "This number($_) is not Natural" };

sub func {
    args my $p => 'Natural';
    say $p;
}

func(p => 3);
# 以下は間違えて呼び出した例です。
# コメントアウトを外して試してみてください。
# func(p => -1);
# func(p => 'aiueo');


# --- 5未満の自然数を定義
subtype 'NaturalLessThanFive'
    => as 'Natural'
    => where { $_ < 5 }
    => message { "$_ is not less than five" };

sub func_with_natural_less_than_five {
    args my $p => 'NaturalLessThanFive';
    say $p;
}

func_with_natural_less_than_five(p => 2);
# func_with_natural_less_than_five(p => 6);


# --- メールアドレス型を定義
subtype 'EmailAddress',
    => as 'Str'
    => where {
        Email::Valid::Loose->address(-address => $_)
    }
    => message { "$_ is not a valid email address" };

sub func_with_email_address {
    args my $email => 'EmailAddress';
    say $email;
}

func_with_email_address(email => 'shibayu36@gmail.com');
# func_with_email_address(email => 'shibayu36');


# --- enumを使った色の定義
enum 'RGBColor' => ['red', 'blue', 'green'];

sub func_with_rgb {
    args my $p => 'RGBColor';
}
func_with_rgb(p => 'red'); # ok
# func_with_rgb(p => 'black'); # error
