use strict;
use warnings;
use utf8;

use feature qw(say);

# このスクリプトはSmart::Argsの基本的な使い方のサンプルコードです
# 2節のコードに対応しています

use URI;
use Smart::Args;

# --- argsで型制約を付ける
sub func {
    args my $number => 'Int',
         my $string => 'Str';

    say(sprintf('%d:%s', $number, $string));
}

func(number => 10, string => 'aiueo');

# 以下は間違えて呼び出した例です。
# コメントアウトを外して試してみてください。
# func(number => 1.5, string => 'aiueo');
# func(number => 10);


# --- default付きで型制約を付ける
sub func_with_default {
    args my $p => { isa => 'Int', default => 10 };

    say($p);
}

func_with_default(p => 5);
func_with_default();


# --- args_posを利用する
sub func_with_args_pos {
    args_pos my $p => 'Int',
             my $q => { isa => 'Str', default => 'aiu' };
    say(sprintf('%d:%s', $p, $q));
}

func_with_args_pos(3, 'abc');


# --- クラスメソッド・インスタンスメソッドに型制約を付ける
package Foo;
use Smart::Args;

sub new {
    my ($class) = @_;
    return bless {}, $class;
}

sub class_method {
    args my $class => 'ClassName',
         my $p     => 'Int';
    say $p;
}


sub instance_method {
    args my $self,
         my $q => 'Str';
    say $q;
}

Foo->class_method(p => 3);
my $foo = Foo->new;
$foo->instance_method(q => 'abc');


# --- 様々な型を利用して制約をかける
package main;
use URI;
use Smart::Args;

sub args_with_uri_and_foo {
    args my $uri => 'URI',
         my $foo => 'Foo';

    say(sprintf('%s:%s', $uri->as_string, $foo));
}
my $uri = URI->new('http://example.com/');
args_with_uri_and_foo(uri => $uri, foo => $foo);

sub args_with_uri_array {
    args my $uris => 'Maybe[ArrayRef[URI]]';

    say join(':', @$uris) if $uris;
}

my $uri1 = URI->new('/hoge');
my $uri2 = URI->new('/fuga');
# 呼び出しは両方成功する
args_with_uri_array(uris => [ $uri1, $uri2 ]);
args_with_uri_array(uris => undef);
