use strict;
use warnings;
use utf8;

use feature qw(say);

# このスクリプトは動的な型制約を応用する例です。
# 4節のコードに対応しています


# --- 仕様を型として定義するサンプル
package Blog;

use Smart::Args;
use Mouse::Util::TypeConstraints;

subtype 'Blog::Title',
    => as 'Str'
    => where {
        3 <= length($_) && length($_) <= 100;
    }
    => message {
        "$_ is not valid blog title";
    };

enum 'Blog::Status' => [qw(public private)];

sub update_blog {
    args my $class   => 'ClassName',
         my $blog_id => 'Int',
         my $title   => 'Blog::Title',
         my $status  => 'Blog::Status';

    # 渡されたデータでのブログデータの更新処理
    # 今回はただprintするだけ
    say sprintf('%d:%s:%s', $blog_id, $title, $status);
}

Blog->update_blog(
    blog_id => 123,
    title   => 'aiu',
    status  => 'public',
);

# 以下は間違えた呼び出し
# Blog->update_blog(
#     blog_id => 'aiu',
#     title   => 'aiu',
#     status  => 'public',
# );
# Blog->update_blog(
#     blog_id => 123,
#     status  => 'public',
# );
# Blog->update_blog(
#     blog_id => 123,
#     title   => 'aiu',
#     status  => 'evil',
# );


# --- 型制約でユーザーの入力を制限する
package main;
use Mouse::Util::TypeConstraints;

# (1)を使うとvalid inputが、(2)を使うとinvalid inputが表示されます

my $is_valid = find_type_constraint('Blog::Title')->check('title'); # (1)
# my $is_valid = find_type_constraint('Blog::Title')->check('a'); # (2)
if ($is_valid) {
    say 'valid input';
}
else {
    say 'invalid input';
}
